#Probe set
###############################################################################################
ngene<-7042                       #number of genes on each slide (with 3 replication)
nslide<-4                         #slides #15,#25,#16,#26.
fraction<-0.4                     #fraction used in lowess SD estimation.
thresh<-2                         #thresholding for filtering.
BLKno <- 48                       #the number of blocks
#######################################Preprocessing####################################################

hp<-read.table(file="H:\\Research\\Probe Set\\data\\finaldata.csv",sep=",",
  row.names=NULL,skip=1)        #import data(there is no "NA" in the finaldata set)
    
ID<-hp[,2]
Block<-hp[,1]
#length(ID)                    #Make sure all the ID's on the 4 slides are the same

Intens<-hp[,c(3,5,7,9)]
Ratios<-hp[,c(4,6,8,10)]

Ratios[,3]<-Ratios[,3]*(-1)           #dye swap,BE CAREFUL ABOUT THE DYE EFFECT       
Ratios[,4]<-Ratios[,4]*(-1) 
               
count<-rep(0,length(ID))
for (j in 1:length(ID)) 
   {
	tmp<-(ID==ID[j])
	count[j]<-sum(tmp)
   }
sum(count==3)                     #select the genes with 3 replications

flag<-(count==3)
ID1<-ID[flag]
Block1<-Block[flag]
Intens<-Intens[flag,]
Ratios<-Ratios[flag,]
dim(Intens)

for (i in 1:nslide)                   #filter the genes with low intensity
   {
	tmp<-Intens[,i]
	tmp[tmp<4.5]<-NA
	Intens[,i]<-tmp
   }
Flag<-(Intens[,1]=="NA")+(Intens[,2]=="NA")+(Intens[,3]=="NA")+(Intens[,4]=="NA")
length(Flag)

Intens<-Intens[Flag==0,]
Ratios<-Ratios[Flag==0,]
ID2<-ID1[Flag==0]
Block2<-Block1[Flag==0]
#dim(Intens)

IntenMed  <- apply(Intens,2,median)   #slide median intensity
IntenMed  <- median(Intens)/IntenMed
Intens <- t(t(Intens)*IntenMed)  #normalized intensities


########################## Quality filtering: Output Geneset#################################
IntenAve<-apply(Intens,1,mean)
IntenSD <-apply((Intens-IntenAve)^2,1,sum)
IntenSD <-sqrt(IntenSD/(nslide-1))
CoefVar <-IntenSD/IntenAve

tmp1<-median(CoefVar)
tmp2<-(quantile(CoefVar,0.75)-quantile(CoefVar,0.25))/1.35    #robust SD

Geneset<-(1:length(ID2))[CoefVar<tmp1+thresh*tmp2]            #filter genes with criteria on CV (see Fan 2004)
length(Geneset)
length(Geneset)/nrow(Intens)

tmp3 <- rep(F, nrow(Intens))
tmp3[Geneset] <- T
Geneset <- tmp3
length(Geneset)
length(ID2)
sum(Geneset)

Intens<-Intens[Geneset,]
Ratios<-Ratios[Geneset,]
ID3<-ID2[Geneset]
Block3<-Block2[Geneset]

count<-rep(0,length(ID3))
for (j in 1:length(ID3))                     #pick out the genes with 3 replication
   {
	tmp<-(ID3==ID3[j])
	count[j]<-sum(tmp)
   }
sum(count==3)

flag<-(count==3)
intens<-Intens[flag,]
ratios<-Ratios[flag,]
Blk.sub<-Block3[flag]
ID.sub<-ID3[flag]
print(dim(ratios))

reponeset<-NULL                                    #gene sets that only one gene resides in the block
for (jj in 1:BLKno) 
{
	tmp<-(Blk.sub==jj)
	if (sum(tmp)==1)
	{
		tmp1<-(1:length(Blk.sub))[tmp==1]
		reponeset<-c(reponeset,tmp1)
	}
}

########################Beginning of the main program########################################################
aa<-dim(ratios)
aa<-aa[1]/3 
croset<-NULL                                                  #generate a random sample for validation test, due to the sample itself, you may get different test statistics
                                                               #values, but the conclusion should be similar, according to our experiment.
for (i in 1:10)                                                 
   {
	 if (i==10) tmp<- sample(c((ceiling(aa/10)*9+1):aa),10)
	 else {tmp<-sample(c((ceiling(aa/10)*(i-1)+1):(ceiling(aa/10)*i)),10)}
	 croset<-c(croset,tmp)
   }
croset<-sort(c(croset,croset+aa,croset+aa*2))

#####data set: intens, ratios; first do the within array normalization, then aggregate 2 slides, considering the dye swap effect.

Blk.sub<-as.character(Blk.sub)
Ratioa<-matrix(0,nrow=length(Blk.sub),ncol=nslide)   #normalized data
norm.res  <- NULL     #output
Ratiob<-matrix(0,nrow=length(Blk.sub),ncol=nslide)   #delete the block effect
Ratiod<-matrix(0,nrow=length(Blk.sub),ncol=nslide)   #intensity effect

for(ii in 1:nslide)
{
  ###############Processing###########	

  Inten<-intens[,ii]
  Ratio<-ratios[,ii]
  number<-ii
  inten<-Inten[-croset]
  ratio<-Ratio[-croset]
  blk.sub<-Blk.sub[-croset]
  ratio<-ratio-median(Ratio)         #Global normalization
  n<-length(ratio)/3


  ######################SLIM Normalization####################################
  exp.level<-rep(0,length(ratio))           #fitted beta
  tmp<-cbind(ratio[1:n],ratio[(n+1):(2*n)],ratio[(2*n+1):(3*n)])
  beta<-apply(tmp,1,mean)   #gene effect
  exp.level[1:n]   <- beta;
  exp.level[(n+1):(2*n)] <- beta;
  exp.level[(2*n+1):(3*n)]<-beta;
  presid<-ratio-exp.level            #partial residuals; substract gene effect

  options(contrasts=c("contr.treatment", "contr.treatment"))
  Blk.eff <- lm(presid ~  blk.sub) #Block eff
  Bfval   <- Blk.eff$fitted.values    #Block fitted value
  presid <- presid - Bfval            #substract Block-effect
  i <- 0; error <- 100;

  while(error > 0.0001 && i < 21)            #loops at most 20 times
    { 
      fitted <- lowess(inten, presid, f=0.2)     #intensity effect
      Sfval  <- approx(fitted, xout=inten)$y      #smoothing fit-value
      Sfval  <- Sfval - mean(Sfval)
      presid <- ratio - Bfval - Sfval
                                 #Partial residuals
      #presid<-Ratio-Sfval                        #ignoring block effect
      beta <- (presid[1:n]+presid[(n+1):(2*n)]+presid[(2*n+1):(3*n)])/3 #RNA effect
      error <- mean((beta - exp.level[1:n])^2); i <- i+1         #gene effect error
      exp.level[1:n] <- beta;
      exp.level[(n+1):(2*n)] <- beta;
      exp.level[(2*n+1):(3*n)] <- beta;
      presid <- ratio - exp.level - Sfval
      #presid<-Ratio-exp.level
      Blk.eff <- lm(presid ~ blk.sub) #Block eff
      error1 <- mean((Bfval - Blk.eff$fitted.value)^2)           #intensity effect error
      Bfval   <- Blk.eff$fitted.values    #Block fitted value
      presid <- ratio - exp.level - Bfval
      print(c(i,error, error1))
     }
  ############## end of fitting the PLS  ########################

  ##### Normalized expression ratio ####
  Ratioa[,ii] <- Ratio-median(Ratio) - approx(fitted, xout=Inten,rule=3)$y
                                                 #substract the intensity effect.
  ######computing block effect
  coeff  <- Blk.eff$coef
  tmp1<- coeff                                   #save a copy
  coeff[2]<- tmp1[10+2]                          #adjust the order of the coefficient
  coeff[3]<- tmp1[20+3]
  coeff[4]<- tmp1[30+4]
  coeff[5:9]<-tmp1[44:BLKno]
  coeff[10:19]<-tmp1[2:11]
  coeff[20:29]<-tmp1[13:22]
  coeff[30:39]<-tmp1[24:33]
  coeff[40:BLKno]<-tmp1[35:43]

  blocks<-Blk.sub                                #start computing block effect
  blocks<-as.numeric(blocks)
  blocks<-blocks+48*(0:(length(blocks)-1))
  tmp <- rep(c(0,coeff[2:BLKno]), length(blocks))
  tmp <- tmp[blocks]                           
  blocks <- tmp  + coeff[1]                      #block effect
   

  Ratioa[,ii] <- Ratioa[,ii] - blocks    #substracting the blocks effect
                                         #Ratioa is now the normalized res
  
  Ratiob[,ii]<-Ratio-median(Ratio)-blocks                  #for plotting purpose
  Ratiod[,ii]<-approx(fitted, xout=Inten,rule=3)$y

  ####### Estimate of Standard Deviations  ##########
  fitted2 <- lowess(Inten, Ratioa[,ii], f=0.2)
  resids  <- Ratioa[,ii] - approx(fitted2, xout=Inten)$y
  resids2  <- resids^2
  fitted3 <- lowess(Inten,resids2, f=0.1)
  SDs <- sqrt(approx(fitted3, xout=Inten)$y)
  Nresults <- matrix(rep(NA,2*length(Ratio)),ncol=2)    #data structure
  dimnames(Nresults) <- list(NULL, c(paste("Nratio",number,sep=""),
        paste("SD",number,sep="")))
  Nresults[,1] <- round(Ratioa[,ii],4)
  Nresults[,2] <- round(SDs,4)


  ################## end with normalization and standard st  ###########

   norm.res <- cbind(norm.res, Nresults)

}


##########################LOWESS Normalization###########################
Ratiolow<-ratios                 #initialize

for (i in 1:nslide)
  {
	Ratiolow[,i]<-lownorm(intens[,i],ratios[,i],Blk.sub,croset)[[1]]
  }


#######################Aggregate SLIM Normalization(very similar with SLIM normalization)##################### 

Ratios1<-Ratiob           #save a copy
Ratios2<-Ratioa
Intens1<-intens
Ratioc<-Ratioa            #save a copy

Ratios1<-as.matrix(Ratios1)
Intens1<-as.matrix(Intens1)
ratios1<-Ratios1[-croset,]
intens1<-Intens1[-croset,]
ratios2<-Ratios2[-croset,]

ratios1<-ratios1-median(ratios1)               #Global normalization
Ratios1<-Ratios1-median(ratios1)               #Global normalization

a<-dim(intens1)
aa<-matrix(1,nrow=a[1],ncol=nslide)

exp.level<-matrix(1,nrow=a[1],ncol=nslide)
n<-a[1]/3

beta<-apply(ratios2,1,mean)
tmp<-beta
tmp<-(tmp[1:n]+tmp[(n+1):(2*n)]+tmp[(2*n+1):(3*n)])/3
beta[1:n]<-tmp
beta[(n+1):(2*n)]<-tmp
beta[(2*n+1):(3*n)]<-tmp

exp.level<-beta*exp.level
presid<-ratios1-exp.level

Sfval<-matrix(0,nrow=a[1],ncol=nslide)       #intensity effect
i<-0; error<-100;

while(error>0.0001 && i<21)                  #loop at most 20 times
  {
	 for (j in 1:nslide)
         {
		fitted<-lowess(intens1[,j],presid[,j],f=0.2)
		Sfval[,j]<-approx(fitted,xout=intens1[,j])$y
		Sfval[,j]<-Sfval[,j]-mean(Sfval[,j])                  #intensity effect
		Ratioc[,j]<-Ratios1[,j]-approx(fitted,xout=Intens1[,j])$y   #fitted value
	   }
	 presid<-ratios1-Sfval
		
	 beta<-apply(presid,1,mean)
	 tmp1<-beta
       tmp1<-(tmp1[1:n]+tmp1[(n+1):(2*n)]+tmp1[(2*n+1):(3*n)])/3
       beta[1:n]<-tmp1
       beta[(n+1):(2*n)]<-tmp1
       beta[(2*n+1):(3*n)]<-tmp1
       error <- mean((beta*aa - exp.level)^2); i <- i+1
       exp.level <- beta*aa;
       
       presid<-ratios1-exp.level
       print(c(i,error))
  }



##########################Validation Test#################################

sd2<-SDreml(Ratioa)

pmatrix<-matrix(0,nrow=8,ncol=nslide)

for (i in 1:nslide) 
  {   
      pmatrix[(2*i-1),1]<-validation(ratios[,i],sd2,croset)[[2]]             #Global normalization
      pmatrix[(2*i-1),2]<-validation(Ratiolow[,i],sd2,croset)[[2]]           #Lowess normalization
      pmatrix[(2*i-1),3]<-validation(Ratioa[,i],sd2,croset)[[2]]		     #SLIM normalization
      pmatrix[(2*i-1),4]<-validation(Ratioc[,i],sd2,croset)[[2]]             #Aggregated SLIM normalization

      pmatrix[(2*i),1]<-validation(ratios[,i],sd2,croset)[[4]]
      pmatrix[(2*i),2]<-validation(Ratiolow[,i],sd2,croset)[[4]]
      pmatrix[(2*i),3]<-validation(Ratioa[,i],sd2,croset)[[4]]
      pmatrix[(2*i),4]<-validation(Ratioc[,i],sd2,croset)[[4]]

  }

